import { BaseNativeObject } from "./model/BaseNativeObject";
import { PowerAuthEncryptionHttpHeader } from "./index";
/**
 * Scope of encryptor.
 */
export type PowerAuthEncryptorScope = 'APPLICATION' | 'ACTIVATION';
export interface PowerAuthCryptogram {
    readonly ephemeralPublicKey?: string;
    readonly encryptedData: string;
    readonly mac: string;
    readonly nonce?: string;
}
export interface PowerAuthEncryptedRequestData {
    readonly cryptogram: PowerAuthCryptogram;
    readonly header: PowerAuthEncryptionHttpHeader;
    readonly decryptor: PowerAuthDecryptor;
}
export interface PowerAuthDecryptor {
    canDecryptResponse(): Promise<boolean>;
    decryptResponse(cryptogram: PowerAuthCryptogram): Promise<string>;
    release(): Promise<void>;
}
export declare class PowerAuthEncryptor extends BaseNativeObject {
    canEncryptRequest(): Promise<boolean>;
    encryptRequest(body: string): Promise<PowerAuthEncryptedRequestData>;
    /**
     * Scope of this encryptor.
     */
    readonly encryptorScope: PowerAuthEncryptorScope;
    /**
     * Instance identifier of PowerAuth instance owning this encryptor.
     */
    private readonly powerAuthInstanceId;
    /**
     *
     */
    private readonly autoreleaseTime;
    /**
     * Construct encryptor object and specify its scope.
     * @param scope Scope of the encryptor.
     * @param powerAuthInstanceId Instance identifier of PowerAuth class owning this encryptor.
     * @param autoreleaseTime Autorelease timeout in milliseconds. The value is used only for the testing purposes, and is ignored in the release build of library.
     */
    constructor(scope: PowerAuthEncryptorScope, powerAuthInstanceId: string, autoreleaseTime?: number);
    protected onCreate(): Promise<string>;
    protected onRelease(objectId: string): Promise<void>;
}
