/*
 * Copyright 2023 Wultra s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import { PowerAuthError, PowerAuthErrorCode } from './PowerAuthError';
import { NativeWrapper } from '../internal/NativeWrapper';
/**
 * Internal class that implements automatic native object re-creation.
 */
var BaseNativeObject = /** @class */ (function () {
    /**
     * Construct object with optional object identifier. If identifier is not provided,
     * then it's created in the next call to `withObjectId()` method.
     * @param objectId Optional object identifier.
     */
    function BaseNativeObject(objectId) {
        if (objectId === void 0) { objectId = undefined; }
        this.objectId = objectId;
    }
    /**
     * Function called when native object needs to be initialized. The derived class must
     * override this function. The function must return native object identifier.
     */
    BaseNativeObject.prototype.onCreate = function () {
        return Promise.reject(new Error("Missing implementation"));
    };
    /**
     * Function called when native object is manually released by the application. The derived class must
     * override this function.
     * @param objectId Native object identifier.
     */
    BaseNativeObject.prototype.onRelease = function (objectId) {
        return Promise.reject(new Error("Missing implementation"));
    };
    /**
     * Function called when native object is re-created after automatic native object cleanup. The default
     * implementation does nothing, so the derived class can override it and implement its own functionality.
     */
    BaseNativeObject.prototype.onAutomaticCleanup = function () {
        // Derived class may implement its own cleanup here.
    };
    /**
     * Acquire native object ID and execute action with this identifier.
     * @param action Action to execute after objectId is acquired.
     * @param recoveringFromError If true, then this is 2nd attempt to execute action after recovery from action.
     * @returns Promise result type.
     */
    BaseNativeObject.prototype.withObjectId = function (action, recoveringFromError) {
        if (recoveringFromError === void 0) { recoveringFromError = false; }
        return __awaiter(this, void 0, void 0, function () {
            var _a, error_1;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _b.trys.push([0, 4, , 5]);
                        if (!!this.objectId) return [3 /*break*/, 2];
                        _a = this;
                        return [4 /*yield*/, this.onCreate()];
                    case 1:
                        _a.objectId = _b.sent();
                        _b.label = 2;
                    case 2: return [4 /*yield*/, action(this.objectId)];
                    case 3: return [2 /*return*/, _b.sent()];
                    case 4:
                        error_1 = _b.sent();
                        if (!recoveringFromError && error_1.code == PowerAuthErrorCode.INVALID_NATIVE_OBJECT) {
                            // Underlying native object has been already destroyed.
                            // We can recover from this situation by resetting identifier and retry the operation
                            this.objectId = undefined;
                            // Call cleanup callback if it's defined
                            this.onAutomaticCleanup();
                            // Now try to repeat action.
                            return [2 /*return*/, this.withObjectId(action, true)];
                        }
                        // Otherwise simply process the exception and report the error.
                        throw NativeWrapper.processException(error_1);
                    case 5: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Initialize native object if it's not initialized yet and convert this object into `RawNativeObject`
     * that is passable to the native code.
     */
    BaseNativeObject.prototype.resolveRawObject = function () {
        return this.withObjectId(function (id) { return Promise.resolve(Object.freeze({ objectId: id })); });
    };
    /**
     * Convert instance of this object into `RawNativeObject` that is passable to the native code.
     * Be aware that this function throws an exception if object identifier is not available.
     */
    BaseNativeObject.prototype.toRawObject = function () {
        if (this.objectId === undefined) {
            throw new PowerAuthError(undefined, "Native object is not initialized", PowerAuthErrorCode.REACT_NATIVE_ERROR);
        }
        return Object.freeze({ objectId: this.objectId });
    };
    BaseNativeObject.prototype.release = function () {
        if (this.objectId) {
            var objId = this.objectId;
            this.objectId = undefined;
            return this.onRelease(objId);
        }
        return Promise.resolve();
    };
    return BaseNativeObject;
}());
export { BaseNativeObject };
