/*
 * Copyright 2023 Wultra s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import { NativeEncryptor } from "../internal/NativeEncryptor";
import { BaseNativeObject } from "./BaseNativeObject";
import { NativeWrapper } from "../internal/NativeWrapper";
/**
 * Class that implements End-To-End encryption. Use `PowerAuth` class to get instnace
 * of properly scoped encryptor.
 */
var PowerAuthEncryptorImpl = /** @class */ (function (_super) {
    __extends(PowerAuthEncryptorImpl, _super);
    /**
     * Construct encryptor object and specify its scope.
     * @param scope Scope of the encryptor.
     * @param powerAuthInstanceId Instance identifier of PowerAuth class owning this encryptor.
     * @param autoreleaseTime Autorelease timeout in milliseconds. The value is used only for the testing purposes, and is ignored in the release build of library.
     */
    function PowerAuthEncryptorImpl(scope, powerAuthInstanceId, autoreleaseTime) {
        if (autoreleaseTime === void 0) { autoreleaseTime = 0; }
        var _this = _super.call(this) || this;
        _this.encryptorScope = scope;
        _this.powerAuthInstanceId = powerAuthInstanceId;
        _this.autoreleaseTime = autoreleaseTime;
        return _this;
    }
    PowerAuthEncryptorImpl.prototype.canEncryptRequest = function () {
        return __awaiter(this, void 0, void 0, function () {
            var _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _b.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, this.withObjectId(function (objjectId) { return NativeEncryptor.canEncryptRequest(objjectId); })];
                    case 1: return [2 /*return*/, _b.sent()];
                    case 2:
                        _a = _b.sent();
                        return [2 /*return*/, false];
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    PowerAuthEncryptorImpl.prototype.encryptRequest = function (body, bodyFormat) {
        var _this = this;
        if (bodyFormat === void 0) { bodyFormat = 'UTF8'; }
        return this.withObjectId(function (objectId) { return __awaiter(_this, void 0, void 0, function () {
            var result;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, NativeEncryptor.encryptRequest(objectId, body, bodyFormat)];
                    case 1:
                        result = _a.sent();
                        return [2 /*return*/, {
                                cryptogram: result.cryptogram,
                                header: result.header,
                                decryptor: new PowerAuthDecryptorImpl(this.encryptorScope, result.decryptorId)
                            }];
                }
            });
        }); });
    };
    PowerAuthEncryptorImpl.prototype.onCreate = function () {
        return NativeEncryptor.initialize(this.encryptorScope, this.powerAuthInstanceId, this.autoreleaseTime);
    };
    PowerAuthEncryptorImpl.prototype.onRelease = function (objectId) {
        return NativeEncryptor.release(objectId);
    };
    return PowerAuthEncryptorImpl;
}(BaseNativeObject));
export { PowerAuthEncryptorImpl };
/**
 * Internal implementation of `PowerAuthDecryptor`
 */
var PowerAuthDecryptorImpl = /** @class */ (function () {
    /**
     * Construct decryptor with original scope and existing native object identifier.
     * @param decryptorScope Original scope used in the encryptor.
     * @param objectId Native object identifier.
     */
    function PowerAuthDecryptorImpl(decryptorScope, objectId) {
        this.decryptorScope = decryptorScope;
        this.objectId = objectId;
    }
    PowerAuthDecryptorImpl.prototype.canDecryptResponse = function () {
        return __awaiter(this, void 0, void 0, function () {
            var error_1;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, this.withObjectId(function (objectId) { return __awaiter(_this, void 0, void 0, function () { return __generator(this, function (_a) {
                                return [2 /*return*/, NativeEncryptor.canDecryptResponse(objectId)];
                            }); }); })];
                    case 1: return [2 /*return*/, _a.sent()];
                    case 2:
                        error_1 = _a.sent();
                        return [2 /*return*/, false];
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    PowerAuthDecryptorImpl.prototype.decryptResponse = function (cryptogram, outputDataFormat) {
        return this.withObjectId(function (objectId) { return NativeEncryptor.decryptResponse(objectId, cryptogram, outputDataFormat); });
    };
    PowerAuthDecryptorImpl.prototype.release = function () {
        return NativeEncryptor.release(this.objectId);
    };
    PowerAuthDecryptorImpl.prototype.withObjectId = function (action) {
        return __awaiter(this, void 0, void 0, function () {
            var error_2;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, action(this.objectId)];
                    case 1: return [2 /*return*/, _a.sent()];
                    case 2:
                        error_2 = _a.sent();
                        throw NativeWrapper.processException(error_2);
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    return PowerAuthDecryptorImpl;
}());
